;+
; NAME:
;	hxt_chsums
; PURPOSE:
;	sums the HXT channel data over all the SC's and maybe minor frames,
;       If full resolution, the data is reformed.
;CATEGORY:
;       Yohkoh HXT
; CALLING SEQUENCE:
;	zz=hxt_chsums(index,data,half_sec=half_sec,tim_sec=tim_sec,$
;                     tim_arr=tim_arr, dt=dt, cps=cps, index_out=index_out,$
;                     data_out=data_out)
; INPUT:
;	data=data structure from rd_hda, do not decompress before using
; OUTPUT:
;       zz= decompressed data summed over subcollimators and maybe minor frames
; KEYWORDS:
;       half_sec= if set, don't sum over the minor frames
;       tim_arr= time array in ANYTIM2INTS structure, corrected fot the
;                HXT time lag, At the START of the accumulation interval
;       tim_sec= time array in seconds starting at zero
;       dt= time intervals, if index is passed in.
;       cps= if set, this will do counts/sec,
;       index_out= index to be passed out, if there is Calibration mode
;                  data this will be different from the index passed in
;       data_out= data to be passed out, if there is Calibration mode
;                  data this will be different from the data passed in
; HISTORY:
;	10-May-93, JM
;       Added reform, index and tim_arr keywords, for plotting convenience,
;                     28-jul-94, jmm
;       Dropped the reform keyword, and index is now a parameter, 1-aug-94.
;       Added correction to deal with the 4.0 second time lag in the hda files,
;                     9/21/94, jmm
;       Added the ability to do non-flare mode data, 2/12/95, jmm
;       Fixed bug which occurs when there is no flare mode data,
;                     and data = 0, 3-feb-96, jmm
;       Puts 64 channel cal mode data into 4 channels, 20-aug-1997, jmm
;       changed, /overwrite option, to pass out index_out and data_out,
;       for cal mode., 6-apr-1999, jmm
;       Changed call from REFORM_HXTCAL to REFORM_HXTCAL2, 8-sep-1999, jmm
;       Added check for non-byte-type data, Probably not good for
;       Cal. mode, 17-mar-2000
;-
FUNCTION Hxt_chsums, index, data, half_sec=half_sec, tim_sec=tim_sec, $
                     tim_arr=tim_arr, dt=dt, cps=cps, Index_out=index1, $
                     Data_out=d1

; Do time correction here
   index1 = fix_hxtindex(index)
   d1 = data
   typ = datatype(d1)
   IF(typ EQ 'BYT') THEN decomp = 1 ELSE decomp = 0
   ndset = N_ELEMENTS(index1)
   nds1 = ndset-1

; For non-flare mode times, data has zeros in the SC's but you
; can get LO channel counts from index.hxt.dhk_data.
   not_flare = where(gt_dp_mode(index1) NE 9)
   IF(N_ELEMENTS(not_flare) EQ ndset OR N_ELEMENTS(data) EQ 1) THEN BEGIN 
      d1 = fltarr(4, 64, 4, ndset) ;All non-flare mode data, jmm 3-feb-96
   ENDIF ELSE BEGIN
;Deal with possible calibration data
      calx = where(index1.hxt.datarectypes EQ 2)
      IF(calx(0) NE -1) THEN BEGIN
         reform_hxtcal2, index1(calx), d1(*, *, *, calx), index1c, d1c
;d1c is a bytarr, not decompressed, decompress here
         d1c = hx_decomp(d1c)
;replace the index and data in the appropriate places
         nd1c = N_ELEMENTS(index1c)
         d1c1 = fltarr(4, 64, nd1c) & d1c1(*) = 0
         d1c2 = fltarr(4, 64, 4, nd1c)

         lo = [1, 8, 16, 32]
         hi = [7, 15, 31, 63]
         FOR i = 0, 3 DO d1c1(i, *, *) =  total(d1c(lo(i):hi(i), *, *), 1)
         FOR j = 0, nd1c-1 DO FOR k = 0, 3 DO $
           d1c2(*, *, k, j) = d1c1(*, *, j)
;         d1c2 = d1c2/4.0        ; something odd is going on
         delvarx, d1c1, d1c
;Here, you'll have to re-compress the data
         d1c2 = hx_comp(d1c2)
         ncalx = where(index1.hxt.datarectypes NE 2)
         IF(ncalx(0) NE -1) THEN BEGIN
            nncalx = N_ELEMENTS(ncalx)
            index1 = [index1(ncalx), index1c]
            n1 = N_ELEMENTS(index1)
            d1o = d1(*, *, *, ncalx)
            d1 = bytarr(4, 64, 4, n1) & d1(*) = 0
            d1(*, *, *, 0:nncalx-1) = d1o
            d1(*, *, *, nncalx:n1-1) = d1c2
;sort by times
            yy = sort(int2secarr(index1))
            index1 = index1(yy)
            d1 = d1(*, *, *, yy)
         ENDIF ELSE BEGIN       ;It's all cal mode.
            index1 = index1c
            d1 = d1c2
         ENDELSE
      ENDIF
;hx_decomp the data...
      IF(decomp) THEN d1 = float(hx_decomp(d1))
   ENDELSE

;Deal with non-flare mode data, need to use the new index
   not_flare = where(gt_dp_mode(index1) NE 9)
   IF(not_flare(0) NE -1) THEN BEGIN
      dhk = float(hx_decomp(index1(not_flare).hxt.dhk_data))/4.0
      FOR k = 0, 3 DO d1(0, *, k, not_flare) = dhk
   ENDIF

;Gotta reset these to be sure
   ndset = N_ELEMENTS(index1)
   nds1 = ndset-1
   calx = where(index1.hxt.datarectypes EQ 2)

;accumulate data...   
   IF KEYWORD_SET(half_sec) THEN BEGIN
; sum subcollimators
      zz = total(d1, 2)         ;pretty neat, jmm 11-feb-95
      zz = reform(zz, 4, 4*ndset, /overwrite)
; dt and tim_sec
;      tim_sec = mk_timarr(index1, 4, ok_map = ok_map)
; Can't use mk_timarr for HXT CAL. MODE, can i??
      dt0 = dprate2sec(index1)
      index1_sec = int2secarr(index1)
      IF(calx(0) NE -1) THEN dt0(calx) = dt0(calx)*4.0
      dt = fltarr(4, ndset)
      tim_sec = dt
      FOR j = 0, nds1 DO BEGIN
         dt(*, j) = dt0(j)/4.0
         tim_sec(*, j) = index1_sec(j)+dt0(j)*[0, 1, 2, 3]/4.0
      ENDFOR
      dt = reform(dt, ndset*4, /overwrite)
      tim_sec = reform(tim_sec, 4*ndset, /overwrite)
      tim_arr = anytim2ints(index1(0), offset = tim_sec)
      IF(KEYWORD_SET(cps)) THEN FOR i = 0, 3 DO zz(i, *) = zz(i, *)/dt
   ENDIF ELSE BEGIN
      zz = fltarr(4, ndset)
;here i need to total over 2 dimensions
      FOR j = 0, nds1 DO FOR i = 0, 3 DO zz(i, j) = total(d1(i, *, *, j))
      dt = dprate2sec(index1)
      IF(calx(0) NE -1) THEN dt(calx) = dt(calx)*4.0
      tim_sec = reform(mk_timarr(index1, 1))
      tim_arr = anytim2ints(index1(0), offset = tim_sec)
      IF(KEYWORD_SET(cps)) THEN FOR i = 0, 3 DO zz(i, *) = zz(i, *)/dt
   ENDELSE
   RETURN, zz
END
